<?php
namespace ZRay;

if (
	class_exists('ZRayPlugin', false)
	&& extension_loaded('Zend OPcache')
	&& php_sapi_name() !== 'cli'
) {

	class OpCache {

		public function collect($context, &$storage) {
			$status = opcache_get_status();
			$config = opcache_get_configuration();

			$storage['Memory Usage'][] = $status['memory_usage'];
			$storage['Interned Strings Usage'][] = $status['interned_strings_usage'];
			$storage['Statistics'][] = $status['opcache_statistics'];
			$storage['Directives'][] = $config['directives'];
			$storage['Version'][] = $config['version'];
			$config['blacklist'] && $storage['Blacklist'][] = $config['blacklist'];

			$scripts = [];
			foreach ($status['scripts'] as $item) {
				$scripts[] = [
					'Name' => basename($item['full_path']),
					'Location' => $item['full_path'],
					'Memory consumption' => round($item['memory_consumption'] / 1024) . " KB",
					'Last Used' => date('Y-m-d H:i:s', $item['last_used_timestamp']),
					'Created' => date("Y-m-d H:i:s", $item['timestamp'])
				];
			}

			$storage['Cached Scripts'][] = $scripts;
		}

		public static function shutdown(): bool {
			return true;
		}
	}

	$zrp = new \ZRayPlugin("OpCache", true);

	register_shutdown_function('ZRay\OpCache::shutdown');

	$opcache = new OpCache();
	$zrp->traceFunction(
		'ZRay\OpCache::shutdown',
		null,
		[$opcache, 'collect']
	);
//	    $zrp->attachAction('clearCache', 'OPcache\shutdown', function(){ return opcache_reset(); });
}
