<?php
namespace ZRay\Wordpress;

class Cache
{
	private $_cache_hits = [];
	private $_cache_misses = [];

	public function wpCacheGetExit($context, &$storage)
	{
		if (!isset($context['locals']['group']) || !isset($context['functionArgs'][0]))
			return;

		$group = $context['locals']['group'];
		$key = $context["functionArgs"][0];
		array_key_exists($group, $this->_cache_hits) || $this->_cache_hits[$group] = [];

		if (isset($context['locals']["found"]) && $context['locals']["found"]) {
			array_key_exists($key, $this->_cache_hits[$group]) || $this->_cache_hits[$group][$key] = 0;
			$this->_cache_hits[$group][$key]++;
		}
	}

	public function beforeExit(&$storage)
	{
		$wpCache = $GLOBALS['wp_object_cache'];

		$data = [];
		$cacheStats = [];

		foreach ($wpCache->cache as $group => $group_items) {
			$group_size = 0;
			$group_hits = 0;

			foreach ($group_items as $group_item_name => $group_item) {
				$group_size += (strlen(serialize($group_item)) / 1024);

				if (isset($this->_cache_hits[$group][$group_item_name])) {
					$group_hits += intval($this->_cache_hits[$group][$group_item_name]);
				}
			}
			$cacheStats[$group] = floatval($group_size);

			$data[] = [
				'Name' => $group,
				'Size' => round($group_size, 2),
				'Hits' => $group_hits
			];
		}
		$storage['Cache Objects'] = $data;

		// General hits/misses data
		asort($cacheStats);
		$cacheStats = array_reverse($cacheStats);
		$total = array_sum($cacheStats);

		$storage['Cache Totals'][] = ['Hits' => $wpCache->cache_hits, 'Misses' => $wpCache->cache_misses, 'Total size' => round($total, 2)];

		$cachePieStats = [];
		foreach ($cacheStats as $name => $value) {
			$cachePieStats[] = ['Name' => $name, 'Count' => round($value, 2)];
		}
		$storage['Cache Stats'] = $cachePieStats;
	}
}