<?php
namespace ZRay\Wordpress;

class Hooks {

	private $_hooks = [];
	private $_core_hooks = [];

	public function beforeExit(&$storage) {
		$storage['hooks'] = $this->_hooks;
		$storage['core hooks'] = $this->_core_hooks;
	}

	public function registerAction($context) {
		empty($context) or $this->registerHook($context, 'Action');
	}

	public function registerFilter($context) {
		empty($context) or $this->registerHook($context, 'Filter');
	}

	private function registerHook($context, $type) {
		if (!defined('WP_PLUGIN_DIR') || !defined('WPMU_PLUGIN_DIR') || !function_exists('get_theme_root'))
			return;

		$hookSource = '';
		$pos = 0;
		if(array_key_exists('calledFromFile', $context)) {
			if (strpos($context['calledFromFile'], realpath(WP_PLUGIN_DIR)) !== false) {
				$pos = strlen(realpath(WP_PLUGIN_DIR)) + 1;
			}
			elseif (strpos($context['calledFromFile'], realpath(WPMU_PLUGIN_DIR)) !== false) {
				$pos = realpath(WPMU_PLUGIN_DIR) + 1;
			}
			elseif (strpos($context['calledFromFile'], realpath(get_theme_root()) !== false)) {
				$pos = realpath(get_theme_root()) + 1;
			}
			if ($pos) {
				$ds_pos = strpos($context['calledFromFile'], DIRECTORY_SEPARATOR, $pos) - 1;
				$hookSource = substr($context['calledFromFile'], $pos, $ds_pos > $pos ? $ds_pos - $pos : null);
			}
		}

		$hookName = strval($context['locals']['hook_name']);
		$hookFunction = $context['locals']['callback'];
		if (is_array($hookFunction)) {
			if (is_string($hookFunction[0])) {
				$hookFunction = $hookFunction[0] . '::' . $hookFunction[1];
			}
			elseif (is_object($hookFunction[0])) {
				$hookFunction = get_class($hookFunction[0]) . '->' . $hookFunction[1];
			}
			else {
				$hookFunction = strval($hookFunction[0]);
			}
		}
		$hookLocation = trim(($context['calledFromFile'] ?? '') . ':' . ($context['calledFromLine'] ?? ''), ':');

		if (empty($hookSource)) {
			$this->_core_hooks[] = [
				'Function' => $hookFunction,
				'Location' => $hookLocation,
				'Name' => $hookName,
				'Type' => $type,
				'Execution time' => floatval($context['durationExclusive']) ?? 0,
				'Priority' => intval($context['locals']['priority']) ?? 10,
			];
		}
		else {
			$this->_hooks[] = [
				'Function' => $hookFunction,
				'Location' => $hookLocation,
				'Name' => $hookName,
				'Type' => $type,
				'Execution time' => floatval($context['durationExclusive']) ?? 0,
				'Source' => $hookSource,
				'Priority' => intval($context['locals']['priority']) ?? 10
			];
		}
	}
}
