<?php
namespace ZRay;

if (class_exists('ZRayPlugin', false)) {

	include 'Wordpress/Cache.php';
	include 'Wordpress/Hooks.php';
	include 'Wordpress/Crons.php';
	include 'Wordpress/Plugins.php';
	include 'Wordpress/WpQuery.php';

	class Wordpress
	{
		private $wordpressCache;
		private $wordpressHooks;

		public function __construct($zre) {

			$this->wordpressCache = new Wordpress\Cache();
			$zre->traceFunction(
				'wp_cache_get',
				null,
				[$this->wordpressCache, 'wpCacheGetExit']
			);

			// Hooks
			$this->wordpressHooks = new Wordpress\Hooks();
			$zre->traceFunction(
				'add_action',
				null,
				[$this->wordpressHooks, 'registerAction']
			);
			$zre->traceFunction(
				'add_filter',
				null,
				[$this->wordpressHooks, 'registerFilter']
			);
		}
		private function getDef(string $name): string {
			return defined($name) ? (constant($name) ? 'On' : 'Off') : 'Undefined';
		}

		public function beforeExit($context, &$storage)
		{
			$storage['Info'][] = [
				'WordPress Version' => $GLOBALS['wp_version'] ?? 'Unknown',
				'Debug Mode (WP_DEBUG)' => $this->getDef('WP_DEBUG'),
				'Debug Log (WP_DEBUG_LOG)' => $this->getDef('WP_DEBUG_LOG'),
				'Script Debug (SCRIPT_DEBUG)' => $this->getDef('SCRIPT_DEBUG'),
				'Template' => function_exists('get_template') ? get_template() : 'Unknown',
				'Template Directory' => function_exists('get_template_directory') ? str_replace('\\', '/', realpath(get_template_directory())) : 'Unknown',
				'Doing Crons' => function_exists('doing_cron') ? (get_transient('doing_cron') ? 'Yes' : 'No') : 'Unknown',
			];

			$this->wordpressCache->beforeExit($storage);

			$this->wordpressHooks->beforeExit($storage);

			$wordpressCrons = new Wordpress\Crons();
			$wordpressCrons->beforeExit($storage);

			$wordpressWpQuery = new Wordpress\WpQuery();
			$wordpressWpQuery->beforeExit($storage);

			$wordpressPlugins = new Wordpress\Plugins();
			$wordpressPlugins->beforeExit($storage);

		}
	}


	$zrp = new \ZRayPlugin('WordPress', true);

	$zrp->traceFunction(
		'wp_initial_constants',
		null,
		function ($context, &$storage) use ($zrp) {
			$zrayWordpress = new Wordpress($zrp);

			$zrp->traceFunction(
				'wp_cache_close',
				null,
				[$zrayWordpress, 'beforeExit']
			);
		}
	);

}